#!/bin/sh

ROOT_DIR="/usr/home"
AVH_DIR="/usr/local/etc/apache22/Includes"
#AVH_DIR="/root/hosting/webuser/tmp"
HS_DIR="/usr/share/skel"
AVH_TEMPLATE="/root/hosting/webuser/vh.template"
PHP_INI_TEMPLATE="/root/hosting/webuser/php.ini.template"
PHP_SH_TEMPLATE="/root/hosting/webuser/php.sh.template"

if [ ! -d $ROOT_DIR ]; then
	echo "Bad path to root home directroy"
	exit 1
fi

if [ ! -d $AVH_DIR ]; then
	echo "Bad path to apache directroy"
	exit 1	
fi

if [ ! -d $HS_DIR ]; then
	echo "Bad path to skel directory"
	exit 1	
fi

if [ ! -f $AVH_TEMPLATE -o \
	 ! -f $PHP_INI_TEMPLATE -o \
	 ! -f $PHP_SH_TEMPLATE ];
then
	echo "Bad path to template files"
	exit 1
fi

user_create()
{
	local res=0
	local user_name="$1"

	if [ `cat /etc/master.passwd | grep $user_name` ]; then
		echo "User already exists"
		return 1
	fi

	pw groupadd $user_name
	if [ $? -ne 0 ]; then $res=$?; fi

	pw useradd $user_name \
      	     -g $user_name \
        	   -d "$ROOT_DIR/$user_name/data" \
          	 -L web_user32 \
	           -s csh
	if [ $? -ne 0 ]; then $res=$?; fi

	if [ $res -ne 0 ]; then
		echo "Can't create user and group"
		return 1
	fi

	return 0
}

user_create_home_skel()
{
	local res=0
	local user_name="$1"

	mkdir -p "$ROOT_DIR/$user_name/data/php-bin"
	if [ $? -ne 0 ]; then $res=$?; fi
	mkdir -p "$ROOT_DIR/$user_name/data/log"
	if [ $? -ne 0 ]; then $res=$?; fi
	mkdir -p "$ROOT_DIR/$user_name/data/tmp"
	if [ $? -ne 0 ]; then $res=$?; fi
	mkdir -p "$ROOT_DIR/$user_name/data/www"
	if [ $? -ne 0 ]; then $res=$?; fi

	if [ $res -ne 0 ]; then
		echo "Can't create home directory"
		if [ -d "$ROOT_DIR/$user_name" ]; then
			rm -rf "$ROOT_DIR/$user_name"
		fi

		return 1
	else
		for file in `ls $HS_DIR`; do
			local new_name=`echo $file | sed -E 's/dot\.(.+)/.\2/gI'`
			cp "$HS_DIR/$file" "$ROOT_DIR/$user_name/$new_name"
		done
	fi 

	return 0
}

user_create_config()
{
	local user_name="$1"
	local user_mail="$2"
	local host_name="$3"
	local host_aliases="$4"

	sed -e "s/\$user/$user_name/g" \
			$PHP_INI_TEMPLATE > "$ROOT_DIR/$user_name/data/php-bin/php.ini"
	if [ $? -ne 0 ]; then
		echo "Can't create php.ini config file"
		return 1
	fi

	sed -e "s/\$user/$user_name/g" \
			$PHP_SH_TEMPLATE > "$ROOT_DIR/$user_name/data/php-bin/php.sh"
	if [ $? -ne 0 ]; then
		echo "Can't create php.sh script file"
		return 1
	fi

	sed -e "s/\$user/$user_name/g" \
			-e "s/\$mail/$user_mail/g" \
			-e "s/\$host/$host_name/g" \
			-e "s/\$aliases/$host_aliases/g" \
			$AVH_TEMPLATE > "$AVH_DIR/$host_name.conf"
	if [ $? -ne 0 ]; then
		echo "Can't create virtual host config file"
		return 1
	fi

	return 0
}

user_set_permissions()
{
	local user_name="$1"

	chown -R $user_name:$user_name "$ROOT_DIR/$user_name"
	chmod 0750 "$ROOT_DIR/$user_name"
	find "$ROOT_DIR/$user_name/" -type d -exec chmod 0750 {} \;
	chown www:www "$ROOT_DIR/$user_name/data/log"
	chmod 0755 "$ROOT_DIR/$user_name/data/log"

	setfacl -m u:www:x "$ROOT_DIR/$user_name"
	setfacl -m u:www:x "$ROOT_DIR/$user_name/data"
	setfacl -m u:www:rx "$ROOT_DIR/$user_name/data/php-bin"
	setfacl -m u::rwx,g::rx,o::---,u:www:rx "$ROOT_DIR/$user_name/data/www"
	setfacl -d -m u::rwx,g::rx,o::---,u:www:rx "$ROOT_DIR/$user_name/data/www"

	chmod 0440 "$ROOT_DIR/$user_name/data/php-bin/php.ini"
	chmod 0550 "$ROOT_DIR/$user_name/data/php-bin/php.sh"
	chflags schg,sunlink "$ROOT_DIR/$user_name/data/php-bin/php.ini"
	chflags schg,sunlink "$ROOT_DIR/$user_name/data/php-bin/php.sh"

	return 0
}

read -p "Username: " user_name
read -p "Admin mail: " user_mail
read -p "Hostname: " host_name
read -p "Host aliases: " host_aliases

if [ ! $user_name ] || [ ! $user_mail ] || [ ! $host_name ] || [ ! "$host_aliases" ]; then
	echo "Please enter correct data"
	exit 1
fi

user_create_home_skel $user_name
if [ $? -ne 0 ]; then
	echo "error: user_create_home_skel"
	exit 1
fi

user_create $user_name
if [ $? -ne 0 ]; then
	echo "error: user_create"
	exit 1
fi

user_create_config $user_name $user_mail $host_name "$host_aliases"
if [ $? -ne 0 ]; then
	echo "error: user_create_config"
	exit 1
fi

user_set_permissions $user_name
if [ $? -ne 0 ]; then
	echo "error: user_ser_permissions"
	exit 1
fi

exit 0

